// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

package frc.robot.commands;

import frc.robot.Constants;
import frc.robot.RobotContainer;
import frc.robot.subsystems.ExampleSubsystem;

import com.ctre.phoenix.motorcontrol.ControlMode;

import edu.wpi.first.math.geometry.Rotation2d;
import edu.wpi.first.math.kinematics.ChassisSpeeds;
import edu.wpi.first.math.kinematics.SwerveDriveKinematics;
import edu.wpi.first.math.kinematics.SwerveModuleState;
import edu.wpi.first.wpilibj2.command.CommandBase;

/** An example command that uses an example subsystem. */
public class ExampleCommand extends CommandBase {
  @SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
  private final ExampleSubsystem m_subsystem;

  private SwerveDriveKinematics m_kinematics;

  /**
   * Creates a new ExampleCommand.
   *
   * @param subsystem The subsystem used by this command.
   */
  public ExampleCommand(ExampleSubsystem subsystem) {
    m_subsystem = subsystem;
    // Use addRequirements() here to declare subsystem dependencies.
    addRequirements(subsystem);

    m_kinematics = new SwerveDriveKinematics(Constants.LF_TRANSLATION, Constants.RF_TRANSLATION, Constants.LB_TRANSLATION, Constants.RB_TRANSLATION);
  }

  // Called when the command is initially scheduled.
  @Override
  public void initialize() {
    m_subsystem.lf_turn.setSelectedSensorPosition(0);
    System.out.println("Cmd Init");
  }

  // Called every time the scheduler runs while the command is scheduled.
  @Override
  public void execute() {


    double x = RobotContainer.s_driver.getLeftY();
    double y = RobotContainer.s_driver.getLeftX();
    double theta = RobotContainer.s_driver.getRightX();

    x = joyExpo(x, Constants.JOY_DRIVE_EXPO);
    y = joyExpo(y, Constants.JOY_DRIVE_EXPO);
    theta = joyExpo(theta, Constants.JOY_DRIVE_EXPO);

    System.out.print(x);
    System.out.print(", ");
    System.out.print(y);
    System.out.print(", ");
    System.out.println(theta); 

    ChassisSpeeds speeds = ChassisSpeeds.fromFieldRelativeSpeeds(x, y, theta, Rotation2d.fromDegrees(0));

    SwerveModuleState[] moduleStates = m_kinematics.toSwerveModuleStates(speeds);

    SwerveModuleState lf_optimized = SwerveModuleState.optimize(
      moduleStates[0],
      Rotation2d.fromDegrees(m_subsystem.lf_turn.getSelectedSensorPosition() / Constants.POD_COUNTS_PER_DEG * 360.0)
    );
    SwerveModuleState rf_optimized = SwerveModuleState.optimize(
      moduleStates[1],
      Rotation2d.fromDegrees(m_subsystem.lf_turn.getSelectedSensorPosition() / Constants.POD_COUNTS_PER_DEG * 360.0)
    );
    SwerveModuleState lb_optimized = SwerveModuleState.optimize(
      moduleStates[2],
      Rotation2d.fromDegrees(m_subsystem.lf_turn.getSelectedSensorPosition() / Constants.POD_COUNTS_PER_DEG * 360.0)
    );
    SwerveModuleState rb_optimized = SwerveModuleState.optimize(
      moduleStates[3],
      Rotation2d.fromDegrees(m_subsystem.lf_turn.getSelectedSensorPosition() / Constants.POD_COUNTS_PER_DEG * 360.0)
    );

    m_subsystem.lf_turn.set(ControlMode.Position, lf_optimized.angle.getDegrees() / 360.0 * Constants.POD_COUNTS_PER_DEG);
    m_subsystem.rf_turn.set(ControlMode.Position, rf_optimized.angle.getDegrees() / 360.0 * Constants.POD_COUNTS_PER_DEG);
    m_subsystem.lb_turn.set(ControlMode.Position, lb_optimized.angle.getDegrees() / 360.0 * Constants.POD_COUNTS_PER_DEG);
    m_subsystem.rb_turn.set(ControlMode.Position, rb_optimized.angle.getDegrees() / 360.0 * Constants.POD_COUNTS_PER_DEG);

    System.out.println(lf_optimized);
    m_subsystem.lf_drive.set(ControlMode.PercentOutput, lf_optimized.speedMetersPerSecond / Constants.DRIVE_MAX_SPEED);
    m_subsystem.rf_drive.set(ControlMode.PercentOutput, rf_optimized.speedMetersPerSecond / Constants.DRIVE_MAX_SPEED);
    m_subsystem.lb_drive.set(ControlMode.PercentOutput, lb_optimized.speedMetersPerSecond / Constants.DRIVE_MAX_SPEED);
    m_subsystem.rb_drive.set(ControlMode.PercentOutput, rb_optimized.speedMetersPerSecond / Constants.DRIVE_MAX_SPEED);
  }

  private double joyExpo(double in, double exp) {
    double out;

    out = Math.exp(Math.abs(in) * exp) - 1;
    out = out / (Math.exp(exp) - 1);
    out = Math.signum(in) * out;

    return out;
  }

  // Called once the command ends or is interrupted.
  @Override
  public void end(boolean interrupted) {}

  // Returns true when the command should end.
  @Override
  public boolean isFinished() {
    return false;
  }
}
